import warnings

from .residuals import compute_contact_point_weights as weight_cp  # noqa: F401

warnings.warn(
    "The 'weight' module is deprecated. Please use "
    "'from nanite.model.residuals import compute_contact_point_weights'!",
    DeprecationWarning)



"""Test of data set functionalities"""
import pathlib

import lmfit
import numpy as np

from nanite import IndentationGroup
from nanite.model import model_sneddon_spherical as hertzSpherical


datadir = pathlib.Path(__file__).resolve().parent / "data"
jpkfile = datadir / "spot3-0192.jpk-force"


def test_app_ret():
    ds = IndentationGroup(jpkfile)
    ar = ds[0]
    ar.apply_preprocessing(["compute_tip_position",
                            "correct_force_offset"])
    idp = ar.estimate_contact_point_index()

    aprid = ~(ar["segment"])
    x = ar["tip position"][aprid]
    y = ar["force"][aprid]
    contact_point = x[idp]

    # crop x and y around contact_point
    distcp = x.shape[0]-idp
    x = x[-3*distcp:]
    y = y[-3*distcp:]

    params = lmfit.Parameters()
    params.add("contact_point", value=contact_point)
    params.add("baseline", value=0, vary=False)
    params.add("E", value=15e3, min=0)
    params.add("nu", value=.5, vary=False)
    params.add("R", value=10e-6, vary=False)

    fit_w = lmfit.minimize(hertzSpherical.residual, params, args=(x, y, True))
    fit_n = lmfit.minimize(hertzSpherical.residual, params, args=(x, y, False))
    # Correctly reproduces fit results in the JPK analysis software
    # with "Vertical Tip Position", "Switchable Baseline Operation",
    # and a spherical indenter with the "Hertz/Sneddon" "Model type".
    # The contact point was calculated through Hertz model and not adjusted
    # before fitting by JPK Software.
    # Set tip radius to 10um.
    E_jpk = 14.94e3
    cp_jpk = 18.04e-6
    assert np.allclose(fit_n.params["E"].value, E_jpk, rtol=1e-1, atol=0)
    assert np.allclose(
        fit_n.params["contact_point"].value, cp_jpk, rtol=4e-3, atol=0)

    if __name__ == "__main__" and False:
        import matplotlib.pylab as plt
        _fig, axes = plt.subplots(2, 1)
        xf = np.linspace(x[0], x[-1], 100)
        axes[0].plot(xf, hertzSpherical.model(
            fit_w.params, xf), label="fit with weights")
        axes[0].plot(xf, hertzSpherical.model(
            fit_n.params, xf), label="fit no weights")
        axes[0].plot(x, y, label="data")
        axes[0].legend()
        axes[0].grid()

        axes[1].plot(x, hertzSpherical.residual(fit_w.params, x, y,
                                                weight_cp=True),
                     label="residuals with weight")
        axes[1].plot(x, hertzSpherical.residual(fit_n.params, x, y,
                                                weight_cp=False),
                     label="residuals no weight")
        axes[1].legend()
        axes[1].grid()

        plt.show()


def test_fit_apret():
    ds = IndentationGroup(jpkfile)
    ar = ds[0]
    ar.apply_preprocessing(["compute_tip_position",
                            "correct_force_offset"])
    ar.fit_model(model_key="sneddon_spher",
                 params_initial=None,
                 x_axis="tip position",
                 y_axis="force",
                 weight_cp=False,
                 segment="retract")

    if __name__ == "__main__":
        import matplotlib.pylab as plt
        _fig, axes = plt.subplots(2, 1)
        axes[0].plot(ar["tip position"], ar["force"], label="data")
        axes[0].plot(ar["tip position"], ar["fit"], label="fit")
        axes[0].legend()
        axes[0].grid()
        plt.show()


if __name__ == "__main__":
    # Run all tests
    loc = locals()
    for key in list(loc.keys()):
        if key.startswith("test_") and hasattr(loc[key], "__call__"):
            loc[key]()



import pathlib

from .indent import Indentation
from .read import load_data


def load_group(path, callback=None):
    """Load indentation data from disk

    Parameters
    ----------
    path: path-like
        Path to experimental data
    callback: callable or None
        Callback function for tracking loading progress

    Returns
    -------
    group: nanite.IndetationGroup
        Indentation group with force-distance data
    """
    path = pathlib.Path(path)
    data = load_data(path, callback=callback)
    grp = IndentationGroup()
    for dd in data:
        grp.append(Indentation(dd))
    grp.path = path
    return grp


class IndentationGroup(object):
    def __init__(self, path=None, callback=None):
        """Group of Indentation

        Parameters
        ----------
        path: str or pathlib.Path or None
            The path to the data file. The data format is determined
            using the extension of the file and the data is loaded
            with the correct method.
        callback: callable or None
            A method that accepts a float between 0 and 1
            to externally track the process of loading the data.
        """
        if path is not None:
            path = pathlib.Path(path)
        self._mmlist = []

        if path is not None:
            self += load_group(path, callback=callback)

        self.path = path

    def __add__(self, grp):
        out = IndentationGroup()
        out._mmlist = self._mmlist + grp._mmlist
        return out

    def __iadd__(self, grp):
        self._mmlist += grp._mmlist
        self.path = None
        return self

    def __iter__(self):
        return iter(self._mmlist)

    def __getitem__(self, idx):
        return self._mmlist[idx]

    def __len__(self):
        return len(self._mmlist)

    def __repr__(self):
        rep = ["IndentationGroup: '{}'".format(self.path)]
        for idnt in self._mmlist:
            rep.append("- {}".format(idnt))
        return "\n".join(rep)

    def get_enum(self, enum):
        """Return the indentation curve with this enum value

        Raises
        ------
        ValueError if multiple curves with the same enum value exist.
        KeyErrir if the enum value is not found
        """
        curves = []
        for item in self._mmlist:
            if item.enum == enum:
                curves.append(item)
        if len(curves) == 0:
            raise KeyError("Could not find dataset with enum {}".format(enum))
        elif len(curves) == 1:
            return curves[0]
        else:
            raise ValueError("Multiple curves with the same enum value exist!")

    def append(self, item):
        """Append an indentation dataset

        Parameters
        ----------
        item: nanite.indent.Indentation
            Force-indentation dataset
        """
        if not isinstance(item, Indentation):
            raise ValueError("`item` must be an instance of `Indentation`!")
        self._mmlist.append(item)

    def index(self, item):
        return self._mmlist.index(item)

    def subgroup_with_path(self, path):
        """Return a subgroup with measurements matching `path`"""
        path = pathlib.Path(path)
        subgroup = IndentationGroup()
        for idnt in self:
            if pathlib.Path(idnt.path).resolve() == path.resolve():
                subgroup.append(idnt)
        subgroup.path = path
        return subgroup



"""Test of data set functionalities"""
import pathlib

import numpy as np
import pytest

from nanite import IndentationGroup


datapath = pathlib.Path(__file__).parent / "data"
jpkfile = datapath / "spot3-0192.jpk-force"


@pytest.mark.filterwarnings('ignore::nanite.smooth.'
                            + 'DoubledSmoothingWindowWarning')
def test_app_ret():
    grp = IndentationGroup(jpkfile)
    idnt = grp[0]
    idnt.apply_preprocessing(["smooth_height"])
    hms = np.array(idnt.data["height (measured, smoothed)"])
    idnt.apply_preprocessing(["compute_tip_position",
                              "smooth_height"])
    hms2 = np.array(idnt.data["height (measured, smoothed)"])
    assert np.all(hms == hms2)
    np.array(idnt.data["tip position (smoothed)"])


def test_tip_sample_separation():
    grp = IndentationGroup(jpkfile)
    idnt = grp[0]
    # This computation correctly reproduces the column
    # "Vertical Tip Position" as it is exported by the
    # JPK analysis software with the checked option
    # "Use Unsmoothed Height".
    idnt.apply_preprocessing(["compute_tip_position"])
    tip = np.array(idnt.data["tip position"])
    assert tip[0] == 2.2803841798545836e-05


def test_correct_app_ret():
    grp = IndentationGroup(jpkfile)
    idnt = grp[0]
    idnt.apply_preprocessing(["compute_tip_position",
                              "correct_split_approach_retract"])
    a = idnt.data["segment"][~(idnt.data["segment"])]
    assert len(a) == 2006


def test_correct_force_offset():
    grp = IndentationGroup(jpkfile)
    idnt = grp[0]
    idnt.apply_preprocessing(["compute_tip_position",
                              "correct_force_offset"])
    idp = idnt.estimate_contact_point_index()
    assert np.allclose(np.average(idnt.data["force"][:idp]), 0)


if __name__ == "__main__":
    # Run all tests
    loc = locals()
    for key in list(loc.keys()):
        if key.startswith("test_") and hasattr(loc[key], "__call__"):
            loc[key]()
